// *************************************
// ** zxspecreckeyboard.h             **
// ** Class Definitions               **
// ** The Recreated Spectrum keyboard **
// *************************************

enum KEYMODES {EMULATOR, STANDARD};
enum KEYSTATES {NORMAL, NORMAL_SHIFTED, NORMAL_SYMBOL, FUNCTION, FUNCTION_SHIFTED, FUNCTION_SYMBOL};

#define NUM_KEYS 40 //40

//debounce countdown value, stop key from being depressed again by accident after release.
#define DEBOUNCE_DELAY 120
#define DEBOUNCE_DELAY_REPEAT 120

class ZxKeyRec {
	private:
	// Reacreated ZX Spectrum Keypress values on Mode A
	byte byKeyDN; // Value of keypress sent - these do not match the keys actual value
	byte byKeyUP; // Value of keypress sent - these do not match the keys actual value

	public:
	ZxKeyRec (byte, byte);

		byte GetKeyDN(){
			//Serial.println(byKeyDN,DEC);
			return byKeyDN;
		}

		byte GetKeyUP(){
			return byKeyUP;
		}

};


class ZxKey  {
	private:

	// Standard Keyboard Characters for Normal PC Use
	// Standard ZX keyboard only in lower case. EMULATOR KEYMODE should use byNormal only.
	byte byNormal;
	// Extra Modes for using keyboard as a normal(ish) PS2 / USB device
	byte byNormalShifted; // Normally Upper Case, to number row arrow keys delete and other
	byte byNormalSymbol; // Red Symbols & Words (replaced by Symbols)
	byte byFunction;	// Standard ZX keyboard but should be used with CTL characters. Numbers = F1 - F10 etc
	byte byGraphicsShifted;	// Standard ZX keyboard but should be used with ALT characters. Numbers = F11 - F12, 5 = home, 6 PGUP etc

	// Monitor Keys last press / activity
	byte byKeyLastValue;
	boolean boKeyDisabled;

	byte KeyValue(KEYSTATES eKeystate) {
		// Get Keyboard character for correct Key state.
		byte byKeyValue = 0;

		switch (eKeystate) {
			case NORMAL_SHIFTED:
				byKeyValue = byNormal;
				// Adjust for Capital Letters
				if (byKeyValue > 96 && byKeyValue < 123){
					byKeyValue = byKeyValue - 32;
				} else {
					byKeyValue = byNormalShifted;
				}
				break;

			case NORMAL_SYMBOL:
				byKeyValue = byNormalSymbol;
				break;

			case FUNCTION:
				byKeyValue = byFunction;
				break;

			default:
				// Standard normal and Emulator mode keyboard.
				// Shift key modifier used later to select functions etc in a ZX81 Emulator.
				byKeyValue = byNormal;
				break;
		}

		return byKeyValue;
	}

	public:
	ZxKey (byte, byte, byte, byte);

		byte KeyPressDetected(KEYSTATES eKeystate, KEYMODES eKeymode){
			// Insure return of 0 if key not reset / debounced
			byte byKeyValue = 0;
			if (boKeyDisabled == false){
				byKeyValue = KeyValue(eKeystate);
				byKeyLastValue = byKeyValue;
				boKeyDisabled = true;
			}
			return byKeyValue;
		}

		byte KeyPressReset(){
			byte byKeyValue = 0;
			if (boKeyDisabled = true) {
				boKeyDisabled = false;
				byKeyValue = byKeyLastValue;
				byKeyLastValue = 0;
			}
			return byKeyValue;
		}

		void KeyDisable(){
			boKeyDisabled = true;
		}

};

// ZxKey constructor
ZxKey::ZxKey (byte Normal, byte NormalShifted, byte NormalSymbol, byte Function){
	// Base Key Values
	byNormal = Normal;
	byNormalShifted = NormalShifted;
	byNormalSymbol = NormalSymbol;
	byFunction = Function;
	//Set starting debounce value
	byKeyLastValue = 0;
	boKeyDisabled = false;
}

// ZxKey constructor
ZxKeyRec::ZxKeyRec (byte KeyDN, byte KeyUP){
	// Recreated Spectrum Key Map Mode 'A'
	byKeyDN = KeyDN;
	byKeyUP = KeyUP;
}

// Defines entire keyboard, includes ZxKey class.
class ZxKeyBoard {
	private:
	// Setup Recreated Spectrum key Code map. Number of Keys should match 'keyCode and keyMap'
	// KeyMap uses Mode 'A' on the Recreated Spectrum
	ZxKeyRec keyCode[NUM_KEYS] =
	{
		// Row 1
		{'a','b'},{'c','d'},{'e','f'},{'g','h'},{'i','j'},{'k','l'},{'m','n'},{'o','p'},{'q','r'},{'s','t'},
		// Row 2
		{'u','v'},{'w','x'},{'y','z'},{'A','B'},{'C','D'},{'E','F'},{'G','H'},{'I','J'},{'K','L'},{'M','N'},
		// Row 3
		{'O','P'},{'Q','R'},{'S','T'},{'U','V'},{'W','X'},{'Y','Z'},{'0','1'},{'2','3'},{'4','5'},{'6','7'},
		// Row 4
		{'8','9'},{'<','>'},{'-','='},{'[',']'},{';',':'},{',','.'},{'/','?'},{'{','}'},{'!','$'},{'%','^'}
	};

	// Setup 4 versions of keyboard states into keyboard.
	// NORMAL, NORMAL_SHIFTED, NORMAL_SYMBOL, FUNCTION
	// Emulation mode should use NORMAL mappings only, as Emulators use keypresses directly to map keys to emulated Spectrums.
	// byte 0 = special or reserved or no function
	ZxKey keyMap[NUM_KEYS] =
	{
		// Row 1
		{'1',KEY_ESC,'!',KEY_F1},{'2',KEY_CAPS_LOCK,'@',KEY_F2},{'3',KEY_PAGE_DOWN,'#',KEY_F3},{'4',KEY_PAGE_UP,'$',KEY_F4},{'5',KEY_LEFT_ARROW,'%',KEY_F5},
		{'6',KEY_DOWN_ARROW,'&',KEY_F6},{'7',KEY_UP_ARROW,'\'',KEY_F7},{'8',KEY_RIGHT_ARROW,'(',KEY_F8},{'9','9',')',KEY_F9},{'0',KEY_BACKSPACE,'_',KEY_DELETE},
		// Row 2
		{'q','q','^',KEY_F10},{'w','w','`',KEY_F11},{'e','e','€',KEY_END},{'r','r','<',KEY_F12},{'t','t','>',KEY_TAB},
		{'y','y','[','y'},{'u','u',']',KEY_PAGE_UP},{'i','i','®',KEY_INSERT},{'o','o',';',KEY_LEFT_ALT},{'p','p','"','©'},
		// Row 3
		{'a','a','~','a'},{'s','s','|','s'},{'d','d','\\',KEY_PAGE_DOWN},{'f','f','{','f'},{'g','g','}',KEY_LEFT_ALT},
		{'h','h','†',KEY_HOME},{'j','j','-','j'},{'k','k','+','k'},{'l','l','=','l'},{KEY_RETURN,KEY_RETURN,KEY_RETURN,KEY_RETURN},
		// Row 4
		{KEY_LEFT_SHIFT,KEY_LEFT_SHIFT,KEY_LEFT_SHIFT,KEY_LEFT_SHIFT},{'z','z',':','z'},{'x','x',163,'x'},{'c','c','?',KEY_RIGHT_CTRL},{'v','v','/','v'},
		{'b','b','*','b'},{'n','n',',','n'},{'m','m','.','m'},{KEY_LEFT_CTRL,KEY_LEFT_CTRL,KEY_LEFT_CTRL,KEY_LEFT_CTRL},{' ',KEY_TAB,' ',KEY_TAB}

	};

	public:
	// Return from ZxKey matching specified state.
	byte byKeyPress(byte byCode, KEYSTATES eKeystate, KEYMODES eKeymode){
		byte byCount = 0;
		while (byCount < NUM_KEYS){
			//Serial.println(byCode,DEC);

			if (keyCode[byCount].GetKeyDN() == byCode){
				return keyMap[byCount].KeyPressDetected(eKeystate, eKeymode);
			}
			byCount++;
		}
		return 0;
	}

	byte KeyPressReset(byte byCode){;
		byte byCount = 0;
		while (byCount < NUM_KEYS){
			if (keyCode[byCount].GetKeyUP() == byCode){
				return keyMap[byCount].KeyPressReset();
			}
			byCount++;
		}
		return 0;
	}

	void KeyDisable(byte byCode){;
		//keyMap[bRow][bCol].KeyDisable();
	}

};
