// ************************************************************************
// **** The Recreated Spectrum USB Keyboard for Leonardo & USB 2 Host. ****
// **** zxspecreckeyboard.ino                                          ****
// ************************************************************************
// ** David Stephenson 2016-10-09  **
// ** Version 1.01                 **
// **********************************

#include <hidboot.h>
#include <usbhub.h>
#include "Arduino.h"
#include "zxspecreckeyboard.h"

// Satisfy the IDE, which needs to see the include statment in the ino too.
#ifdef dobogusinclude
#include <spi4teensy3.h>
#include <SPI.h>
#endif

// Class taken from USB 2 host keyboard demo
class KbdRptParser : public KeyboardReportParser
{
	protected:
		void OnKeyDown	(uint8_t mod, uint8_t key);
		byte byKeyPressed = 0;

	public:
		byte getKeyPressed();
};

void KbdRptParser::OnKeyDown(uint8_t mod, uint8_t key)
{
	uint8_t c = OemToAscii(mod, key);
	if (c){
		byKeyPressed=(byte)c;
	}
}

byte KbdRptParser::getKeyPressed(){
	return byKeyPressed;
}



// ************************
// *** Global Variables ***
// ************************

// *** Setup Global Variables for keyboard
ZxKeyBoard MyKeyboard;

//KEYMODES eMyKeyMode = EMULATOR;
KEYMODES eMyKeyMode = STANDARD; // Start keyboard in PS2 Mode
KEYSTATES eMyKeyState = NORMAL; // Normal / base keyboard layout


// *** USB Host Setup ***
USB     Usb;
//USBHub     Hub(&Usb);
HIDBoot<USB_HID_PROTOCOL_KEYBOARD>    HidKeyboard(&Usb);
uint32_t next_time;
KbdRptParser Prs;



void setup()
{
	Serial.begin( 115200 );
#if !defined(__MIPSEL__)
	//while (!Serial); // Wait for serial port to connect - used on Leonardo, Teensy and other boards with built-in USB CDC serial connection
#endif
	//Serial.println("Start");

	if (Usb.Init() == -1)
	Serial.println("OSC did not start.");

	delay( DEBOUNCE_DELAY * 4 );

	next_time = millis() + 5000;
	Keyboard.begin();

}

void loop()
{

	int x=0;
	byte byKeyZXAct = 0; // Hold Last Key Action Value Detected from Recreated ZX Spectrum
	byte byKeyZXVal = 0; // Hold Returned ASCII Value of Key

	// Special Action keys
	bool boKeysShifted = false;
	bool boKeysSymboled = false;

	HidKeyboard.SetReportParser(0, &Prs);

	while(x==0){
		Usb.Task();

		if(Prs.getKeyPressed() != byKeyZXAct){
			byKeyZXAct = Prs.getKeyPressed();
			byKeyZXVal = MyKeyboard.byKeyPress(byKeyZXAct, eMyKeyState, eMyKeyMode);
			if(byKeyZXVal == 0){
				// Action Key Release Codes
				byKeyZXVal = MyKeyboard.KeyPressReset(byKeyZXAct);
				if(byKeyZXVal != 0){
					if (eMyKeyMode == STANDARD){
						// PS2 Keyboard Mode - STANDARD
						// Check for Special Action Keys - SHIFT And SYMBOL SHIFT
						if ( byKeyZXVal == KEY_LEFT_SHIFT || byKeyZXVal == KEY_LEFT_CTRL){
							if(eMyKeyState == FUNCTION){
								boKeysShifted = false;
								boKeysSymboled = false;
								eMyKeyState = NORMAL;
							} else {
								switch (byKeyZXVal){
									case KEY_LEFT_SHIFT:
										boKeysShifted = false;
									break;

									case KEY_LEFT_CTRL:
										boKeysSymboled = false;
									break;
								}
								eMyKeyState = NORMAL;
							}

							byKeyZXVal = 0;
						}
					} else {
						// Keyboard in Emulator Mode
						if(byKeyZXVal == KEY_LEFT_SHIFT) boKeysShifted = false;
						if(byKeyZXVal == KEY_LEFT_CTRL) boKeysSymboled = false;
					}

					//Serial.println("Released");
					//Serial.println((char)byKeyZXVal);
					//if  (byKeyZXVal == 163) Keyboard.release(KEY_LEFT_ALT);
					if (byKeyZXVal) Keyboard.release(byKeyZXVal);
				}
			} else {
				// Action Key Press Codes
				if (eMyKeyMode == STANDARD){
					// PS2 Keyboard Mode - STANDARD
					// Check for Special Action Keys
					switch (byKeyZXVal){
						case KEY_LEFT_SHIFT:
							boKeysShifted = true;
							byKeyZXVal = 0;
						break;

						case KEY_LEFT_CTRL:
							boKeysSymboled = true;
							byKeyZXVal = 0;
					}

					if(byKeyZXVal == 0){
						if(boKeysShifted) eMyKeyState = NORMAL_SHIFTED;
						if(boKeysSymboled) eMyKeyState = NORMAL_SYMBOL;
						if(boKeysSymboled && boKeysShifted) eMyKeyState = FUNCTION;
					}

					if(eMyKeyState == FUNCTION && byKeyZXVal == KEY_RETURN){
						eMyKeyMode = EMULATOR;
						eMyKeyState = NORMAL;
						boKeysSymboled = false;
						boKeysShifted = false;
						byKeyZXVal = 0;
					}
				} else {
					// Keyboard in Emulator Mode
					if(byKeyZXVal == KEY_LEFT_SHIFT) boKeysShifted = true;
					if(byKeyZXVal == KEY_LEFT_CTRL) boKeysSymboled = true;
					if(boKeysShifted && boKeysSymboled && byKeyZXVal == KEY_RETURN){
						eMyKeyMode = STANDARD;
						eMyKeyState = NORMAL;
						boKeysShifted = false;
						boKeysSymboled = false;
						Keyboard.release(KEY_LEFT_SHIFT);
						Keyboard.release(KEY_LEFT_CTRL);
						byKeyZXVal = 0;
					}
				}

				//Serial.println("Pressed");
				//Serial.println((char)byKeyZXVal);
				//if  (byKeyZXVal == 163) Keyboard.press(KEY_LEFT_ALT);
				if (byKeyZXVal) Keyboard.press(byKeyZXVal);
			}

		}
	}
}

